﻿using System.Linq;
using Ato.CD.Inbound.Shared;
using System.Collections.Generic;
using VaTS;
using Ato.EN.IntegrationServices.CodeGenerationCTR;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationLSSCRRYBCK;

namespace Ato.CD.Inbound.CTR202102
{
    internal class CrossFormValidatorLSSCRRYBCK : ICrossFormValidator
    {
        private CTR2021 ParentDocument { get; }

        private LSSCRRYBCK2021 ChildDocument { get; }

        internal CrossFormValidatorLSSCRRYBCK(CTR2021 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (LSSCRRYBCK2021)childDocument.ConsumedReport;
        }
        
        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();

            VRATOCTR500260(response);
            VRATOCTR500262(response);

            return response;
        }

        #region VR.ATO.CTR.500260

        private void VRATOCTR500260(List<ProcessMessageDocument> response)
        {
            /*  VR.ATO.CTR.500260
            When a Loss Carry Back schedule is present, then Loss carry back amendment indicator must be set to true
    
            Legacy Rule Format:
            IF (COUNT(SCHEDULE = "LSSCRRYBCK") = 1) AND [CTR414] <> TRUE
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            COUNT(LSSCRRYBCK) = 1 AND ^CTR414 <> TRUE
    
            Data Elements:
    
            ^CTR414 = CTR:RP:rvctc3.02.xx:Tax.Losses.CarryBackAmendment.Indicator
            */
            bool assertion = ChildDocument != null && ParentDocument.CTR414 != true;
            if (assertion)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.500260",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Loss carry back amendment indicator must be set to true",
                    Location = "/xbrli:xbrl/tns:Tax.Losses.CarryBackAmendment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CTR.500260" } },
                };
                response.Add(processMessage);
            }
        }

        #endregion

        #region VR.ATO.CTR.500262

        private void VRATOCTR500262(List<ProcessMessageDocument> response)
        {
            /*  VR.ATO.CTR.500262
            When a Loss Carry Back schedule is provided, the Business entity tax file number must match the one provided on the main form
    
            Legacy Rule Format:
            IF (COUNT(SCHEDULE="LSSCRRYBCK") = 1) AND [LSSCRRYBCK1] <> [CTR8]
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            COUNT(LSSCRRYBCK) = 1 AND ^LSSCRRYBCK1 <> ^CTR8
    
            Data Elements:
    
            ^CTR414 = CTR:RP:rvctc3.02.xx:Tax.Losses.CarryBackAmendment.Indicator
            */
            bool assertion = ChildDocument != null && ChildDocument.LSSCRRYBCK1 != ParentDocument.CTR8;
            if (assertion)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.500262",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"When a Loss Carry Back schedule is provided, the Business entity tax file number must match the one provided on the main form",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CTR.500262" } },
                };
                response.Add(processMessage);
            }
        }

        #endregion
    }
}