using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using Ato.EN.IntegrationServices.CodeGenerationRS;
using Ato.EN.IntegrationServices.CodeGenerationTRTAMI;
using VaTS;
using DataContracts;

namespace Ato.CD.Inbound.TRTAMI202102
{
    internal class CrossFormValidatorRS : ICrossFormValidator
    {
        private TRTAMI2021 ParentDocument { get; }
        private RS2018 ChildDocument { get; }

        internal CrossFormValidatorRS(TRTAMI2021 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (RS2018)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATORS000137(response);
            VRATORS000138(response);
            VRATOGEN402009(response);
            VRATORS000150(response);

            return response;
        }
        #region VR.ATO.RS.000137
        /* 
        IF (PARENT RETURN <> "IITR") AND ([RS25] > 52)  
        RETURN VALIDATION MESSAGE
        ENDIF
        */
        private void VRATORS000137(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.RS25.GetValueOrDefault() > 52;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.RS.000137",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Number of weeks property rented must not exceed 52",                    
                    Location = "/xbrli:xbrl/tns:RealEstateProperty.RentalWeeks.Count",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.RS.000137" } }
                });
            }
        }
        #endregion

        #region VR.ATO.RS.000138
        /* 
        IF (PARENT RETURN <> ""IITR"") AND ([RS27] > 52)  
           RETURN VALIDATION MESSAGE
        ENDIF
        */
        private void VRATORS000138(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.RS27.GetValueOrDefault() > 52;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.RS.000138",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Number of weeks property was available for rent this year cannot exceed 52 weeks",
                    Location = "/xbrli:xbrl/tns:RealEstateProperty.RentalWeeksAvailable.Count",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.RS.000138" } }
                });
            }
        }
        #endregion 

        #region VR.ATO.GEN.402009
        /* VR.ATO.GEN.402009
            IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                RETURN VALIDATION MESSAGE
            ENDIF 
        */
        private void VRATOGEN402009(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = "Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                });
            }
        }
        #endregion // VR.ATO.GEN.402009

        #region VR.ATO.RS.000150
        /*   
            Rule ID: VR.ATO.RS.000150
            
            I IF (PARENT RETURN <> "TRTAMI" AND [RS14] <> PARENT RETURN(pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text (IN TUPLE(xbrli\organisationame2.xx.xx:OrganisationNameDetails) WHERE (pyde.xx.xx:OrganisationNameDetails.OrganisationalType.Code = "MN") AND (pyde.xx.xx:OrganisationNameDetails.Currency.Code = "C")))) 
            OR (PARENT RETURN = "TRTAMI" AND [RS14] <> PARENT RETURN:RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text)
               RETURN VALIDATION MESSAGE
            ENDIF
           [RS14] = RS:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
        */
        private void VRATORS000150(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.RS14.Trim() != ParentDocument.TRTAMI10.Trim();
              
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.RS.000150",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Entity name must be the same as the name on the main form",
                    Location = "/xbrli:xbrl/rs:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.RS.000150" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion 
    }
}
